import base64
import os
import sqlite3
from pathlib import Path as Pathlib_path
import click
from flask import current_app, g
from flask.cli import with_appcontext

static_images = os.path.dirname(os.path.abspath(__file__)) + "/static/images/styles/small/"
save_parent_folder = "/storage/emulated/0/Music/EisenRadio"
script_path = os.path.dirname(__file__)


def convert_ascii(file_name):
    with open(file_name, "rb") as reader:
        img_bytes = reader.read()
        img_ascii = render_picture(img_bytes, 'encode')
    return img_ascii


def render_picture(byte_data, de_enc):
    render_pic = ''
    if de_enc == 'encode':
        render_pic = base64.b64encode(byte_data).decode('ascii')
    if de_enc == 'decode':
        render_pic = base64.b64decode(byte_data).decode('ascii')
    return render_pic


def get_db():
    if 'db' not in g:
        g.db = sqlite3.connect(
            current_app.config['DATABASE'],
            detect_types=sqlite3.PARSE_DECLTYPES
        )
        g.db.row_factory = sqlite3.Row

    return g.db


def close_db(e=None):
    db = g.pop('db', None)

    if db is not None:
        db.close()


def init_db():
    db = get_db()

    with current_app.open_resource((os.path.join(script_path, 'schema.sql'))) as file_schema:
        db.executescript(file_schema.read().decode('utf8'))
    make_db_from_schema()


@click.command('init-db')
@with_appcontext
def init_db_command():
    """Clear the existing data and create new tables."""
    init_db()
    click.echo('Initialized the database.')


def init_app(app):
    app.teardown_appcontext(close_db)
    app.cli.add_command(init_db_command)


def make_db_from_schema():
    # # # # custom filled db # # #
    connection = ''
    is_snap_device = 'SNAP' in os.environ
    if not is_snap_device:
        connection = sqlite3.connect((os.path.join(script_path, 'database.db')))
    if is_snap_device:
        # not overwrite db from old version, where an existing 'database.db' was copied
        snap_db = Pathlib_path(os.path.join(os.environ["SNAP_USER_COMMON"], 'pre_configured.db'))
        if snap_db.is_file():
            return
        if not snap_db.is_file():
            connection = sqlite3.connect(str(snap_db))

    with open((os.path.join(script_path, 'schema.sql'))) as f:
        connection.executescript(f.read())

    cur = connection.cursor()
    cur.execute("INSERT INTO eisen_intern (browser_open,statistics,commercials) VALUES (?,?,?)", (str(1), 1, 1))

    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('classic',
         'http://37.251.146.169:8000/streamHD',
         save_parent_folder,
         'ro - hq stream',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-brown_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('goa_psi',
         'http://amoris.sknt.ru:8004/stream',
         save_parent_folder,
         'ru - Trance Psychedelic Experimental',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-black_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('time_machine',
         'http://98.211.68.9:8765/listen',
         save_parent_folder,
         'usa - Classic Old Time Radio, Sci Fi, Comedy, Drama',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-blue_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('korean_pop',
         'http://167.114.64.181:8818/stream',
         save_parent_folder,
         '',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-violet_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('nerd-sound-tracks',
         'http://167.114.210.232:8259/stream',
         save_parent_folder,
         'sound tracks of japanese anime videos and games',
         "image/jpeg",
         convert_ascii(static_images + "mixer-tempelhof-airport.jpg"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('BLUES_UK',
         'http://149.255.59.3:8232/stream',
         save_parent_folder,
         '',
         "image/jpeg",
         convert_ascii(static_images + "mixer-reload.jpg"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('relax',
         'http://5.39.86.44:8006/stream-mp3-CoolCoffee_autodj',
         save_parent_folder,
         '',
         "image/jpeg",
         convert_ascii(static_images + "mixer-construction-site.jpg"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('Nachtflug',
         'http://85.195.88.149:11810/sid=1',
         save_parent_folder,
         'added sid=1, http://85.195.88.149:11810/sid=1',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-red_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('hm',
         'https://hirschmilch.de:7001/prog-house.mp3',
         save_parent_folder,
         '',
         "image/png",
         convert_ascii(static_images + "radio3d-style-whiteneongelb_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('Boradio',
         'http://www.boradio.nl:8000/aacp',
         save_parent_folder,
         'aacp file format, vlc can read it',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-neongreen_120x200.png"))
        )

    connection.commit()
    connection.close()


def empty_db_from_schema():
    # # # # custom filled db # # #
    connection = ''
    is_snap_device = 'SNAP' in os.environ
    if not is_snap_device:
        connection = sqlite3.connect((str(os.path.join(script_path, 'database.db'))))
    if is_snap_device:
        connection = sqlite3.connect((str(os.path.join(os.environ["SNAP_USER_COMMON"], 'pre_configured.db'))))

    with open((os.path.join(script_path, 'schema.sql'))) as f:
        connection.executescript(f.read())

    cur = connection.cursor()
    cur.execute("INSERT INTO eisen_intern (browser_open,statistics,commercials) VALUES (?,?,?)", (str(1), 1, 1))
    connection.commit()
    connection.close()



from logging import getLogger
from eisenradio import wsgi
from waitress import serve

logger = getLogger('waitress')
logger.setLevel("ERROR")


app = wsgi.app
print('\n Python WSGI server "Waitress" \n')
if __name__ == "__main__":
    serve(app, host='localhost', port=5050)



from eisenradio.api import ghettoApi


def measure_meta():

    request_time = ''
    request_suffix = ''
    play_station = ''
    request_icy_genre = ''
    request_icy_name = ''
    station_id = ''
    request_icy_view_id = ''
    request_icy_br = ''

    try:
        # listen key pressed
        for radio_id, btn_down in ghettoApi.listen_dict.items():
            if btn_down:
                station_id = radio_id
                play_station = ghettoApi.radios_in_view_dict[radio_id]
                request_time = ghettoApi.ghetto_measure_dict[play_station + ',request_time']

                content_type = ghettoApi.ghetto_measure_dict[play_station + ',suffix']
                if content_type == 'audio/aacp' or content_type == 'application/aacp':
                    request_suffix = 'aacp'
                if content_type == 'audio/aac':
                    request_suffix = 'aac'
                if content_type == 'audio/ogg' or content_type == 'application/ogg':
                    request_suffix = 'ogg'
                if content_type == 'audio/mpeg':
                    request_suffix = 'mp3'
                if content_type == 'audio/x-mpegurl' or content_type == 'text/html':
                    request_suffix = 'm3u'
    except KeyError:
        pass
    try:
        request_icy_genre = ghettoApi.ghetto_measure_dict[play_station + ',icy_genre']
        request_icy_name = ghettoApi.ghetto_measure_dict[play_station + ',icy_name']
        request_icy_view_id = str(station_id)
        request_icy_br = ghettoApi.ghetto_measure_dict[play_station + ',icy_br']
    except KeyError:
        pass
    if not request_time:
        request_time = '---'
        request_suffix = '---'

    return request_time, request_suffix, request_icy_genre, request_icy_name, request_icy_view_id, request_icy_br



import json
import unittest
from eisenradio import create_app  # __init__
from eisenradio.instance.config_apfac import write_config, remove_config

# POST: app.post('/endpoint', data=params)
# GET:  app.get('/endpoint', query_string=params)


class TestRouteHome(unittest.TestCase):

    """ radio DB table id ( radio 0-9  ) """
    radio_to_delete = int(3)

    def setUp(self):

        print(f"\n\t --------------------test-setUp-----START----\n")
        # remove_config()
        write_config('test')    # write_config('test', True)  for an empty db, other test setup
        print(f"\t --------------------test-setUp-----END-----\n")

    def test_route_home(self):

        from eisenradio.lib.eisdb import status_read_status_set
        from eisenradio.eisenhome import routes as home_routes

        app = create_app(True)  # True is test

        print('\n ... Begin: def test_route_home()')

        print('\napp.name:\t' + app.name)
        print(f'app.config[DEBUG]:\t {app.config["DEBUG"]}')
        print(f'app.config[TESTING]:\t {app.config["TESTING"]}')
        print(f'app.config[DATABASE]:\t {app.config["DATABASE"]}\n')

        web = app.test_client()
        rv = web.get('/')

        assert rv.status_code == 200
        assert '<title> Your Eisen Radio! </title>' in rv.data.decode('utf-8')

        print(""" test Blueprint 'eisenHOME'  """)

        assert home_routes.eisenhome_bp.template_folder == 'bp_home_templates'
        assert home_routes.eisenhome_bp.static_url_path == '/bp_home_static'

        print(""" /setcookiedark """)

        rv = web.get('/setcookiedark')
        assert rv.status_code == 200
        assert 'Eisenkekse sind die besten' in rv.data.decode('utf-8')

        print(""" /getcookiedark (json style) """)

        rv = web.get('/getcookiedark')
        data = json.loads(rv.get_data(as_text=True))
        assert rv.status_code == 200
        assert data['darkmode'] == 'darkmode'

        print(""" /getcookiedark (flask client.cookie_jar style) """)

        cookie = next(
            (cookie for cookie in web.cookie_jar if cookie.name == "eisen-cookie"),
            None
        )
        assert cookie is not None
        assert cookie.value == "darkmode"

        print(""" /delcookiedark """)

        rv = web.post('/delcookiedark',
                      headers={"X-Requested-With": "XMLHttpRequest"},
                      follow_redirects=True, )
        cookie = rv.headers.getlist('Set-Cookie')
        cookie_val = cookie[0]  # nice one row list :(
        cook_list = cookie_val.split(';')
        assert cook_list[0] == 'eisen-cookie='  # cookie value is set to 0
        assert rv.status_code == 200
        assert b'necesito nuevas cookies' in rv.data

        timer = int(9)
        rv = web.post('/index_posts_combo',
                      data=dict(time_record_select_all=timer),
                      follow_redirects=True,
                      content_type='application/x-www-form-urlencoded',
                      )
        data = json.loads(rv.get_data(as_text=True))
        assert data == timer
        assert rv.status_code == 200

        print(""" /getcookiedark """)

        rv = web.post('/index_posts_percent',
                      follow_redirects=True,
                      content_type='application/x-www-form-urlencoded',
                      )
        data = json.loads(rv.get_data(as_text=True))
        assert rv.status_code == 200
        assert data['result'] == 0

        print(""" /display_info """)

        import eisenradio.eisenhome.eishome as eisen_radio
        eisen_radio.active_displays = {2: 'OVAN 오반 - I Need You 어떻게 지내',
                                       3: 'Henrik Freischlader Band - Take The Blame',
                                       6: 'Yoko Kanno and The Seatbelts - Waltz for Zizi'}
        rv = web.get('/display_info',
                     follow_redirects=True,
                     content_type='application/x-www-form-urlencoded',
                     )
        data = json.loads(rv.get_data(as_text=True))
        assert rv.status_code == 200
        assert data['result'] == "2=OVAN 오반 - I Need You 어떻게 지내,3=Henrik Freischlader Band - Take The Blame,6=Yoko " \
                                 "Kanno and The Seatbelts - Waltz for Zizi,"

        print(""" /page_flash """)

        expected_flash_message = 'Count down timer ended all activities. App restart recommended!'
        rv = web.get('/page_flash')
        parsed = rv.data.decode(encoding="utf-8")
        assert rv.status_code == 200
        assert expected_flash_message in parsed
        assert eisen_radio.combo_master_timer == 0  # master timer recording
        assert eisen_radio.progress_master_percent == 0

        print(""" /<int:id>/delete  must FAIL, active Listen """)

        eisen_radio.status_listen_btn_dict[self.radio_to_delete] = 1
        eisen_radio.status_record_btn_dict[self.radio_to_delete] = 0

        """ query failed, item not in store? only reading here, deletion is going via route /<int:id>/delete """
        rv_db = status_read_status_set(False, 'posts', 'title', str(self.radio_to_delete))
        # print(rv_db)
        assert rv_db != "column not in table posts, status_read_status_set"
        rv = web.post('/' + str(self.radio_to_delete) + '/delete', follow_redirects=True)  # redirect to index
        parsed = rv.data.decode(encoding="utf-8")
        assert rv.status_code == 200
        assert 'Radio is active. No deletion.' in parsed  # flash message

        print(""" /<int:id>/delete  must FAIL, active Record """)

        eisen_radio.status_listen_btn_dict[self.radio_to_delete] = 0
        eisen_radio.status_record_btn_dict[self.radio_to_delete] = 1
        rv = web.post('/' + str(self.radio_to_delete) + '/delete', follow_redirects=True)  # redirect to index
        parsed = rv.data.decode(encoding="utf-8")
        assert rv.status_code == 200
        assert 'Radio is active. No deletion.' in parsed  # flash message

        print(""" /<int:id>/delete  DELETE action """)

        eisen_radio.status_listen_btn_dict[self.radio_to_delete] = 0
        eisen_radio.status_record_btn_dict[self.radio_to_delete] = 0
        rv = web.post('/' + str(self.radio_to_delete) + '/delete', follow_redirects=True)  # DELETE
        parsed = rv.data.decode(encoding="utf-8")
        assert rv.status_code == 200
        assert 'was successfully deleted' in parsed

        print(""" query MUST fail, radio not in store! """)
        rv_db = status_read_status_set(False, 'posts', 'title', str(self.radio_to_delete))
        assert rv_db == "column not in table posts, status_read_status_set"
        print(""" column deleted """)

        print('--- fin ----')

    def tearDown(self):
        print(f"\n\t --------------------test-tearDown -----START----\n")
        remove_config()
        print(f"\n\t --------------------test-tearDown -----END----\n")



class Api:
    def init_app(self, app):
        self.config = app.config


class GhettoApi:
    def init_listen(self, listen_dict):
        self.listen_dict = listen_dict

    def init_radios_in_view(self, radios_in_view_dict):
        self.radios_in_view_dict = radios_in_view_dict

    def init_ghetto_measurements(self, ghetto_measure_dict):
        self.ghetto_measure_dict = ghetto_measure_dict


api = Api()
ghettoApi = GhettoApi()



import setuptools
import os
import sys

with open("README.md", "r") as fh:
    long_description = fh.read()

# ###############################################
pack_name="eisenradio"
pack_version="0.0.4"
pack_description="Play radio. Record radio. Style your App."

if sys.platform=='win32':
    print('----> sys.platform ' + sys.platform)
    INSTALL_REQUIRES = [
            'setuptools',
            'flask_cors',
            'wmi',
            'pywin32',
            'winshell',
            'winreg',
            'configparser',
            'requests',
            'urllib3'
        ]
else:
    INSTALL_REQUIRES = [
        'setuptools',
        'flask_cors',
        'configparser',
        'requests',
        'urllib3'
    ]
PYTHON_REQUIRES = '>=3.6'


setuptools.setup(

    name=pack_name, # project name /folder
    version=pack_version,
    author="René Horn",
    author_email="rene_horn@gmx.net",
    description=pack_description,
    long_description=long_description,
    license='gpl-3.0',
    long_description_content_type="text/markdown",
    url="",
    include_package_data=True,
    packages=setuptools.find_packages(),
        install_requires=INSTALL_REQUIRES,
    classifiers=[
    # How mature is this project? Common values are
    # https://packaging.python.org/guides/distributing-packages-using-setuptools/
    #   3 - Alpha
    #   4 - Beta
    #   5 - Production/Stable
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
        "Topic :: Multimedia :: Sound/Audio :: Capture/Recording",
    ],
    python_requires=PYTHON_REQUIRES,
)
# https://stackoverflow.com/questions/20288711/post-install-script-with-python-setuptools
# from setuptools import setup
def _post_install():
    import site
    import getpass
    import datetime

    module_dir=''
    print('----------------------------- POST INSTALL ----------------------------------')
    username=getpass.getuser()
    print(os.path.dirname(os.path.abspath(__file__)))
    print('sitepack ' + site.getusersitepackages())

    setup_dir=os.path.dirname(os.path.abspath(__file__))
    for path in sys.path:
        if not path.find(pack_name + '-' + pack_version)==-1:
            if not path==setup_dir:
                module_dir=path + '\\' + pack_name
                print('Module directory: ' + module_dir)

    if sys.platform=='win32':
        print('----> sys.platform ' + sys.platform)
        from win32com.client import Dispatch
        import winreg
        shell = Dispatch('WScript.Shell')
        # Read location of Windows desktop folder from registry
        regName = 'Desktop'
        regPath = r'Software\Microsoft\Windows\CurrentVersion\Explorer\User Shell Folders'
        desktopFolder = os.path.normpath(get_reg(regName,regPath))
        print('desktop ' + desktopFolder)

        # Path to location of link file
        link_name = pack_name + '.lnk'
        pathLink = os.path.join(desktopFolder, link_name)
        if os.path.exists(pathLink):
            print("Remove existing shortcut at %s" % repr(pathLink))
            os.unlink(pathLink)

        shortcut = shell.CreateShortCut(pathLink)
        print(pathLink)
        shortcut.WorkingDirectory = module_dir
        shortcut.Targetpath = module_dir + '\\app.py'
        print(module_dir + '\\app.py')
        print(module_dir + '\\eisenradio\\static\\images\\eisen.ico')
        shortcut.IconLocation = module_dir + '\\eisenradio\\static\\images\\eisen.ico'
        shortcut.save()

def get_reg(name,path):
    # Read variable from Windows Registry
    # From https://stackoverflow.com/a/35286642
    import winreg
    try:
        registry_key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, path, 0,
                                       winreg.KEY_READ)
        value, regtype = winreg.QueryValueEx(registry_key, name)
        winreg.CloseKey(registry_key)
        return value
    except WindowsError:
        return None

def modules_directory(mod):
    if os.path.exists(mod):
        return mod
	
_post_install()



def test_sum():
    assert sum([1, 2, 3]) == 6, "Should be 6"

def test_sum_tuple():
    assert sum((1, 2, 2)) == 6, "Should be 6"

if __name__ == "__main__":
    test_sum()
    test_sum_tuple()
    print("Everything passed")
